/*
******************************************************************************

   OVL debugging macros

   Copyright (C) 2001 Christoph Egger	[Christoph_Egger@t-online.de]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _OVL_INTERNAL_DEBUG_H
#define _OVL_INTERNAL_DEBUG_H


#include <ggi/internal/ovl.h>

#include <stdio.h>
#include <stdarg.h>
#include <ggi/types.h>
#include <ggi/gg.h>



__BEGIN_DECLS

/* Exported variables */
#ifdef BUILDING_OVL
extern uint32     _ovlDebugState;
extern int        _ovlDebugSync;
#else
IMPORTVAR uint32  _ovlDebugState;
IMPORTVAR int     _ovlDebugSync;
#endif
__END_DECLS


/* Debugging types
 * bit 0 is reserved! */

#define OVLDEBUG_CORE		(1<<1)	/*   2 */
#define OVLDEBUG_MODE		(1<<2)	/*   4 */
#define OVLDEBUG_COLOR		(1<<3)	/*   8 */
#define OVLDEBUG_DRAW		(1<<4)	/*  16 */
#define OVLDEBUG_MISC		(1<<5)	/*  32 */
#define OVLDEBUG_LIBS		(1<<6)	/*  64 */
#define OVLDEBUG_EVENTS		(1<<7)	/* 128 */

#define OVLDEBUG_ALL	0xffffffff


#ifdef DEBUG
#define LIB_ASSERT(x)	if (x) {} else { ggDPrintf(__FILE__, __LINE__, __PRETTY_FUNCTION__, "assertion \"%s\" failed.", #x); }
#else
#define LIB_ASSERT(x)	do{}while(0)
#endif


#ifdef __GNUC__

#ifdef DEBUG
#define OVLDPRINT(args...)	   if (_ovlDebugState) { ggDPrintf(_ovlDebugSync, "LibOVL",args); }
#define OVLDPRINT_CORE(args...)   if (_ovlDebugState & OVLDEBUG_CORE) { ggDPrintf(_ovlDebugSync,"LibOVL",args); }
#define OVLDPRINT_MODE(args...)   if (_ovlDebugState & OVLDEBUG_MODE) { ggDPrintf(_ovlDebugSync,"LibOVL",args); }
#define OVLDPRINT_COLOR(args...)  if (_ovlDebugState & OVLDEBUG_COLOR) { ggDPrintf(_ovlDebugSync,"LibOVL",args); }
#define OVLDPRINT_DRAW(args...)   if (_ovlDebugState & OVLDEBUG_DRAW) { ggDPrintf(_ovlDebugSync,"LibOVL",args); }
#define OVLDPRINT_MISC(args...)   if (_ovlDebugState & OVLDEBUG_MISC) { ggDPrintf(_ovlDebugSync,"LibOVL",args); }
#define OVLDPRINT_LIBS(args...)   if (_ovlDebugState & OVLDEBUG_LIBS) { ggDPrintf(_ovlDebugSync,"LibOVL",args); }
#define OVLDPRINT_EVENTS(args...) if (_ovlDebugState & OVLDEBUG_EVENTS) { ggDPrintf(_ovlDebugSync,"LibOVL",args); }
#else /* DEBUG */
#define OVLDPRINT(args...)		do{}while(0)
#define OVLDPRINT_CORE(args...)	do{}while(0)
#define OVLDPRINT_MODE(args...)	do{}while(0)
#define OVLDPRINT_COLOR(args...)	do{}while(0)
#define OVLDPRINT_DRAW(args...)	do{}while(0)
#define OVLDPRINT_MISC(args...)	do{}while(0)
#define OVLDPRINT_LIBS(args...)	do{}while(0)
#define OVLDPRINT_EVENTS(args...)	do{}while(0)
#endif /* DEBUG */

#else /* __GNUC__ */

__BEGIN_DECLS

static inline void OVLDPRINT(const char *form,...)
{
#ifdef DEBUG
	if (_ovlDebugState) {
		va_list args;

		fprintf(stderr, "OVL: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ovlDebugSync) fflush(stderr);
	}
#endif
}

static inline void OVLDPRINT_CORE(const char *form,...)
{
#ifdef DEBUG
	if (_ovlDebugState & OVLDEBUG_CORE) {
		va_list args;

		fprintf(stderr, "OVL: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ovlDebugSync) fflush(stderr);
	}
#endif
}

static inline void OVLDPRINT_MODE(const char *form,...)
{
#ifdef DEBUG
	if (_ovlDebugState & OVLDEBUG_MODE) {
		va_list args;

		fprintf(stderr, "OVL: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ovlDebugSync) fflush(stderr);
	}
#endif
}

static inline void OVLDPRINT_COLOR(const char *form,...)
{
#ifdef DEBUG
	if (_ovlDebugState & OVLDEBUG_COLOR) {
		va_list args;

		fprintf(stderr, "OVL: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ovlDebugSync) fflush(stderr);
	}
#endif
}

static inline void OVLDPRINT_DRAW(const char *form,...)
{
#ifdef DEBUG
	if (_ovlDebugState & OVLDEBUG_DRAW) {
		va_list args;

		fprintf(stderr, "OVL: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ovlDebugSync) fflush(stderr);
	}
#endif
}

static inline void OVLDPRINT_MISC(const char *form,...)
{
#ifdef DEBUG
	if (_ovlDebugState & OVLDEBUG_MISC) {
		va_list args;

		fprintf(stderr, "OVL: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ovlDebugSync) fflush(stderr);
	}
#endif
}

static inline void OVLDPRINT_LIBS(const char *form,...)
{
#ifdef DEBUG
	if (_ovlDebugState & OVLDEBUG_LIBS) {
		va_list args;

		fprintf(stderr, "OVL: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ovlDebugSync) fflush(stderr);
	}
#endif
}

static inline void OVLDPRINT_EVENTS(const char *form,...)
{
#ifdef DEBUG
	if (_ovlDebugState & OVLDEBUG_EVENTS) {
		va_list args;

		fprintf(stderr, "OVL: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ovlDebugSync) fflush(stderr);
	}
#endif
}

__END_DECLS

#endif /* __GNUC__ */

#ifdef DEBUG
#define OVL_ASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"OVL:%s:%d: INTERNAL ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#define OVL_APPASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"OVL:%s:%d: APPLICATION ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#else /* DEBUG */
#define OVL_ASSERT(x,str)	do{}while(0)
#define OVL_APPASSERT(x,str)	do{}while(0)
#endif /* DEBUG */

#ifdef DEBUG
# define OVLD0(x)	x
#else
# define OVLD0(x)	/* empty */
#endif

#ifdef OVLDLEV
# if OVLDLEV == 1
#  define OVLD1(x)	x
#  define OVLD2(x)	/* empty */
#  define OVLD3(x)	/* empty */
# elif OVLDLEV == 2
#  define OVLD1(x)	x
#  define OVLD2(x)	x
#  define OVLD3(x)	/* empty */
# elif OVLDLEV > 2
#  define OVLD1(x)	x
#  define OVLD2(x)	x
#  define OVLD3(x)	x
# endif
#else
# define OVLD1(x)	/* empty */
# define OVLD2(x)	/* empty */
# define OVLD3(x)	/* empty */
#endif

#endif /* _OVL_INTERNAL_DEBUG_H */
